@@ -1,5 +1,5 @@
 # This file is part of the pyMOR project (https://www.pymor.org).
-# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# Copyright pyMOR developers and contributors. All rights reserved.
 # License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import numpy as np
@@ -64,7 +64,7 @@ def gram_schmidt(A, product=None, return_R=False, atol=1e-13, rtol=1e-13, offset
         # first calculate norm
         initial_norm = A[i].norm(product)[0]
 
-        if initial_norm < atol:
+        if initial_norm <= atol:
             logger.info(f"Removing vector {i} of norm {initial_norm}")
             remove.append(i)
             continue
@@ -91,7 +91,7 @@ def gram_schmidt(A, product=None, return_R=False, atol=1e-13, rtol=1e-13, offset
                 old_norm, norm = norm, A[i].norm(product)[0]
 
                 # remove vector if it got too small
-                if norm < rtol * initial_norm:
+                if norm <= rtol * initial_norm:
                     logger.info(f"Removing linearly dependent vector {i}")
                     remove.append(i)
                     break
