@@ -1,5 +1,5 @@
 # This file is part of the pyMOR project (https://www.pymor.org).
-# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# Copyright pyMOR developers and contributors. All rights reserved.
 # License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import numpy as np
@@ -70,14 +70,16 @@ def lyap_lrcf_solver_options(lradi_tol=1e-10,
                                              'tol': wachspress_tol}}}}
 
 
-def solve_lyap_lrcf(A, E, B, trans=False, options=None):
+def solve_lyap_lrcf(A, E, B, trans=False, cont_time=True, options=None):
     """Compute an approximate low-rank solution of a Lyapunov equation.
 
-    See :func:`pymor.algorithms.lyapunov.solve_lyap_lrcf` for a
-    general description.
+    See
 
-    This function uses the low-rank ADI iteration as described in
-    Algorithm 4.3 in :cite:`PK16`.
+    - :func:`pymor.algorithms.lyapunov.solve_cont_lyap_lrcf`
+
+    for a general description.
+
+    This function uses the low-rank ADI iteration as described in Algorithm 4.3 in :cite:`PK16`.
 
     Parameters
     ----------
@@ -88,18 +90,20 @@ def solve_lyap_lrcf(A, E, B, trans=False, options=None):
     B
         The operator B as a |VectorArray| from `A.source`.
     trans
-        Whether the first |Operator| in the Lyapunov equation is
-        transposed.
+        Whether the first |Operator| in the Lyapunov equation is transposed.
+    cont_time
+        Whether the continuous- or discrete-time Lyapunov equation is solved.
+        Only the continuous-time case is implemented.
     options
-        The solver options to use (see
-        :func:`lyap_lrcf_solver_options`).
+        The solver options to use (see :func:`lyap_lrcf_solver_options`).
 
     Returns
     -------
     Z
-        Low-rank Cholesky factor of the Lyapunov equation solution,
-        |VectorArray| from `A.source`.
+        Low-rank Cholesky factor of the Lyapunov equation solution, |VectorArray| from `A.source`.
     """
+    if not cont_time:
+        raise NotImplementedError
     _solve_lyap_lrcf_check_args(A, E, B, trans)
     options = _parse_options(options, lyap_lrcf_solver_options(), 'lradi', None, False)
     logger = getLogger('pymor.algorithms.lradi.solve_lyap_lrcf')
