@@ -1,8 +1,8 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
-""" This module provides some operators for continuous finite element discretizations."""
+"""This module provides some operators for continuous finite element discretizations."""
 
 from functools import partial
 
@@ -25,28 +25,28 @@ from pymor.vectorarrays.numpy import NumpyVectorSpace
 
 
 LagrangeShapeFunctions = {
-        line: {1: [lambda X: 1 - X[..., 0],
-                   lambda X: X[..., 0]]},
-        square: {1: [lambda X: (1 - X[..., 0]) * (1 - X[..., 1]),
-                     lambda X: (1 - X[..., 1]) * (X[..., 0]),
-                     lambda X:     (X[..., 0]) * (X[..., 1]),
-                     lambda X:     (X[..., 1]) * (1 - X[..., 0])]},
-        triangle: {1: [lambda X: 1 - X[..., 0] - X[..., 1],
-                       lambda X: X[..., 0],
-                       lambda X: X[..., 1]]},
-            }
+    line: {1: [lambda X: 1 - X[..., 0],
+               lambda X: X[..., 0]]},
+    square: {1: [lambda X: (1 - X[..., 0]) * (1 - X[..., 1]),
+                 lambda X: (1 - X[..., 1]) * (X[..., 0]),
+                 lambda X:     (X[..., 0]) * (X[..., 1]),
+                 lambda X:     (X[..., 1]) * (1 - X[..., 0])]},
+    triangle: {1: [lambda X: 1 - X[..., 0] - X[..., 1],
+                   lambda X: X[..., 0],
+                   lambda X: X[..., 1]]},
+}
 
 LagrangeShapeFunctionsGrads = {
-        line: {1: np.array(([-1.],
-                            [1., ]))},
-        square: {1: lambda X: np.array(([X[..., 1] - 1., X[..., 0] - 1.],
-                                        [1. - X[..., 1], - X[..., 0]],
-                                        [X[..., 1], X[..., 0]],
-                                        [-X[..., 1], 1. - X[..., 0]]))},
-        triangle: {1: np.array(([-1., -1.],
-                                [1., 0.],
-                                [0., 1.]))}
-        }
+    line: {1: np.array(([-1.],
+                        [1., ]))},
+    square: {1: lambda X: np.array(([X[..., 1] - 1., X[..., 0] - 1.],
+                                    [1. - X[..., 1], - X[..., 0]],
+                                    [X[..., 1], X[..., 0]],
+                                    [-X[..., 1], 1. - X[..., 0]]))},
+    triangle: {1: np.array(([-1., -1.],
+                            [1., 0.],
+                            [0., 1.]))},
+}
 
 
 def CGVectorSpace(grid, id='STATE'):
@@ -54,7 +54,7 @@ def CGVectorSpace(grid, id='STATE'):
 
 
 class L2ProductFunctionalP1(NumpyMatrixBasedOperator):
-    """Linear finite element functional representing the inner product with an L2-|Function|.
+    """Linear functional representing the inner product with an L2-|Function|.
 
     Parameters
     ----------
@@ -111,7 +111,7 @@ class L2ProductFunctionalP1(NumpyMatrixBasedOperator):
 
 
 class BoundaryL2ProductFunctional(NumpyMatrixBasedOperator):
-    """Linear finite element functional representing the inner product with an L2-|Function| on the boundary.
+    """Linear functional representing the inner product with an L2-|Function| on the boundary.
 
     Parameters
     ----------
@@ -167,7 +167,7 @@ class BoundaryL2ProductFunctional(NumpyMatrixBasedOperator):
 
 
 class BoundaryDirichletFunctional(NumpyMatrixBasedOperator):
-    """Linear finite element functional for enforcing Dirichlet boundary values.
+    """Linear functional for enforcing Dirichlet boundary values.
 
     Parameters
     ----------
@@ -201,7 +201,7 @@ class BoundaryDirichletFunctional(NumpyMatrixBasedOperator):
 
 
 class L2ProductFunctionalQ1(NumpyMatrixBasedOperator):
-    """Bilinear finite element functional representing the inner product with an L2-|Function|.
+    """Bilinear functional representing the inner product with an L2-|Function|.
 
     Parameters
     ----------
@@ -232,7 +232,8 @@ class L2ProductFunctionalQ1(NumpyMatrixBasedOperator):
         g = self.grid
         bi = self.boundary_info
 
-        # evaluate function at all quadrature points -> shape = (g.size(0), number of quadrature points)
+        # evaluate function at all quadrature points ->
+        #   shape = (g.size(0), number of quadrature points)
         F = self.function(g.centers(0), mu=mu)
 
         # evaluate the shape functions at the quadrature points on the reference
@@ -665,10 +666,11 @@ class AdvectionOperatorP1(NumpyMatrixBasedOperator):
                  dirichlet_clear_columns=False, dirichlet_clear_diag=False,
                  solver_options=None, name=None):
         assert grid.reference_element(0) in {triangle, line}, 'A simplicial grid is expected!'
-        assert advection_function is None \
-            or (isinstance(advection_function, Function)
-                and advection_function.dim_domain == grid.dim
-                and advection_function.shape_range == (grid.dim,))
+
+        advection_function = advection_function or ConstantFunction(np.ones((grid.dim,)), grid.dim)
+        assert isinstance(advection_function, Function)
+        assert advection_function.dim_domain == grid.dim
+        assert advection_function.shape_range == (grid.dim,)
         self.__auto_init(locals())
         self.source = self.range = CGVectorSpace(grid)
 
@@ -955,10 +957,12 @@ def discretize_stationary_cg(analytical_problem, diameter=None, domain_discretiz
     preassemble
         If `True`, preassemble all operators in the resulting |Model|.
     mu_energy_product
-        If not `None`, |parameter values| for which to assemble the symmetric part of the |Operator| of the resulting
-        |Model| `fom` (ignoring the advection part). Thus, assuming no advection and a symmetric diffusion tensor,
-        `fom.products['energy']` is equal to `fom.operator.assemble(mu)`, except for the fact that the former has
-        cleared Dirichlet rows and columns, while the latter only has cleared Dirichlet rows).
+        If not `None`, |parameter values| for which to assemble the symmetric part of the
+        |Operator| of the resulting |Model| `fom` (ignoring the advection part). Thus,
+        assuming no advection and a symmetric diffusion tensor, `fom.products['energy']`
+        is equal to `fom.operator.assemble(mu)`, except for the fact that the former has
+        cleared Dirichlet rows and columns, while the latter only
+        has cleared Dirichlet rows).
 
     Returns
     -------
@@ -972,7 +976,6 @@ def discretize_stationary_cg(analytical_problem, diameter=None, domain_discretiz
             :unassembled_m:  In case `preassemble` is `True`, the generated |Model|
                              before preassembling operators.
     """
-
     assert isinstance(analytical_problem, StationaryProblem)
     assert grid is None or boundary_info is not None
     assert boundary_info is None or grid is not None
@@ -1073,7 +1076,7 @@ def discretize_stationary_cg(analytical_problem, diameter=None, domain_discretiz
             if mu_energy_product:
                 eLi += [RobinBoundaryOperator(grid, boundary_info, robin_data=p.robin_data)]
         else:
-            Li += [RobinBoundaryOperator(grid, boundary_info, robin_data=p.robin_data, name=f'robin')]
+            Li += [RobinBoundaryOperator(grid, boundary_info, robin_data=p.robin_data, name='robin')]
             if mu_energy_product:
                 eLi += [RobinBoundaryOperator(grid, boundary_info, robin_data=p.robin_data)]
             coefficients.append(1.)
@@ -1097,23 +1100,23 @@ def discretize_stationary_cg(analytical_problem, diameter=None, domain_discretiz
     if p.neumann_data is not None and boundary_info.has_neumann:
         if isinstance(p.neumann_data, LincombFunction):
             Fi += [BoundaryL2Functional(grid, -ne, boundary_info=boundary_info,
-                                      boundary_type='neumann', dirichlet_clear_dofs=True, name=f'neumann_{i}')
+                                        boundary_type='neumann', dirichlet_clear_dofs=True, name=f'neumann_{i}')
                    for i, ne in enumerate(p.neumann_data.functions)]
             coefficients_F += list(p.neumann_data.coefficients)
         else:
             Fi += [BoundaryL2Functional(grid, -p.neumann_data, boundary_info=boundary_info,
-                                      boundary_type='neumann', dirichlet_clear_dofs=True)]
+                                        boundary_type='neumann', dirichlet_clear_dofs=True)]
             coefficients_F.append(1.)
 
     if p.robin_data is not None and boundary_info.has_robin:
         if isinstance(p.robin_data[0], LincombFunction):
             Fi += [BoundaryL2Functional(grid, rob * p.robin_data[1], boundary_info=boundary_info,
-                                      boundary_type='robin', dirichlet_clear_dofs=True, name=f'robin_{i}')
+                                        boundary_type='robin', dirichlet_clear_dofs=True, name=f'robin_{i}')
                    for i, rob in enumerate(p.robin_data[0].functions)]
             coefficients_F += list(p.robin_data[0].coefficients)
         else:
             Fi += [BoundaryL2Functional(grid, p.robin_data[0] * p.robin_data[1], boundary_info=boundary_info,
-                                      boundary_type='robin', dirichlet_clear_dofs=True)]
+                                        boundary_type='robin', dirichlet_clear_dofs=True)]
             coefficients_F.append(1.)
 
     if p.dirichlet_data is not None and boundary_info.has_dirichlet:
@@ -1128,7 +1131,7 @@ def discretize_stationary_cg(analytical_problem, diameter=None, domain_discretiz
     F = LincombOperator(operators=Fi, coefficients=coefficients_F, name='rhsOperator')
 
     if grid.reference_element in (triangle, square):
-        visualizer = PatchVisualizer(grid=grid, bounding_box=grid.bounding_box(), codim=2)
+        visualizer = PatchVisualizer(grid=grid, codim=2)
     elif grid.reference_element is line:
         visualizer = OnedVisualizer(grid=grid, codim=1)
     else:
@@ -1169,7 +1172,9 @@ def discretize_stationary_cg(analytical_problem, diameter=None, domain_discretiz
                     outputs.append(BoundaryL2Functional(grid, v[1], dirichlet_clear_dofs=False).H)
         if len(outputs) > 1:
             from pymor.operators.block import BlockColumnOperator
+            from pymor.operators.constructions import NumpyConversionOperator
             output_functional = BlockColumnOperator(outputs)
+            output_functional = NumpyConversionOperator(output_functional.range) @ output_functional
         else:
             output_functional = outputs[0]
     else:
@@ -1198,7 +1203,6 @@ def discretize_stationary_cg(analytical_problem, diameter=None, domain_discretiz
 
     data = {'grid': grid, 'boundary_info': boundary_info}
 
-
     if preassemble:
         data['unassembled_m'] = m
         m = preassemble_(m)
@@ -1209,8 +1213,10 @@ def discretize_stationary_cg(analytical_problem, diameter=None, domain_discretiz
 def discretize_instationary_cg(analytical_problem, diameter=None, domain_discretizer=None, grid_type=None,
                                grid=None, boundary_info=None, num_values=None, time_stepper=None, nt=None,
                                preassemble=True):
-    """Discretizes an |InstationaryProblem| with a |StationaryProblem| as stationary part
-    using finite elements.
+    """Finite Element discretization of an |InstationaryProblem|.
+
+    Discretizes an |InstationaryProblem| with a |StationaryProblem| as the
+    stationary part using finite elements.
 
     Parameters
     ----------
@@ -1255,7 +1261,6 @@ def discretize_instationary_cg(analytical_problem, diameter=None, domain_discret
             :unassembled_m:  In case `preassemble` is `True`, the generated |Model|
                              before preassembling operators.
     """
-
     assert isinstance(analytical_problem, InstationaryProblem)
     assert isinstance(analytical_problem.stationary_part, StationaryProblem)
     assert grid is None or boundary_info is not None
@@ -1265,6 +1270,11 @@ def discretize_instationary_cg(analytical_problem, diameter=None, domain_discret
 
     p = analytical_problem
 
+    if p.stationary_part.dirichlet_data is not None and 't' in p.stationary_part.dirichlet_data.parameters:
+        # we choose both mass and operator to be invertible.
+        # this leads to wrong results when the dirichlet values depend on time.
+        raise NotImplementedError('Time-dependent Dirichlet values not supported.')
+
     m, data = discretize_stationary_cg(p.stationary_part, diameter=diameter, domain_discretizer=domain_discretizer,
                                        grid_type=grid_type, grid=grid, boundary_info=boundary_info)
 
