@@ -1,14 +1,13 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import numpy as np
 
 from pymor.core.cache import CacheableObject
 from pymor.operators.constructions import induced_norm
-from pymor.parameters.base import ParametricObject, Mu
+from pymor.parameters.base import Parameters, ParametricObject, Mu
 from pymor.tools.frozendict import FrozenDict
-from pymor.tools.deprecated import Deprecated
 
 
 class Model(CacheableObject, ParametricObject):
@@ -31,6 +30,8 @@ class Model(CacheableObject, ParametricObject):
         `True` if the model describes a linear problem.
     products
         Dict of inner product operators associated with the model.
+    order
+        Dimension of the `solution_space`.
     """
 
     solution_space = None
@@ -38,7 +39,7 @@ class Model(CacheableObject, ParametricObject):
     linear = False
     products = FrozenDict()
 
-    def __init__(self, products=None, error_estimator=None, visualizer=None,
+    def __init__(self, dim_input=0, products=None, error_estimator=None, visualizer=None,
                  name=None):
         products = FrozenDict(products or {})
         if products:
@@ -46,11 +47,18 @@ class Model(CacheableObject, ParametricObject):
                 setattr(self, f'{k}_product', v)
                 setattr(self, f'{k}_norm', induced_norm(v))
 
+        self.parameters_internal = {'input': dim_input}
+
         self.__auto_init(locals())
 
+    @property
+    def order(self):
+        return self.solution_space.dim
+
     def _compute(self, solution=False, output=False, solution_d_mu=False, output_d_mu=False,
                  solution_error_estimate=False, output_error_estimate=False,
-                 output_d_mu_return_array=False, mu=None, **kwargs):
+                 output_d_mu_return_array=False, output_error_estimate_return_vector=False,
+                 mu=None, **kwargs):
         return {}
 
     def _compute_solution(self, mu=None, **kwargs):
@@ -104,10 +112,7 @@ class Model(CacheableObject, ParametricObject):
         |NumPy array| with the computed output or a dict which at least
         must contain the key `'output'`.
         """
-        if not getattr(self, 'output_functional', None):
-            return np.zeros(len(solution), 0)
-        else:
-            return self.output_functional.apply(solution, mu=mu).to_numpy()
+        return self.output_functional.apply(solution, mu=mu).to_numpy()
 
     def _compute_solution_d_mu_single_direction(self, parameter, index, solution, mu=None, **kwargs):
         """Compute the partial derivative of the solution w.r.t. a parameter index
@@ -153,7 +158,7 @@ class Model(CacheableObject, ParametricObject):
         return sensitivities
 
     def _compute_output_d_mu(self, solution, mu=None, return_array=False, **kwargs):
-        """compute the gradient w.r.t. the parameter of the output functional
+        """Compute the gradient w.r.t. the parameter of the output functional
 
         Parameters
         ----------
@@ -173,17 +178,18 @@ class Model(CacheableObject, ParametricObject):
         U_d_mus = self._compute_solution_d_mu(solution, mu)
         gradients = [] if return_array else {}
         for (parameter, size) in self.parameters.items():
-            array = np.empty(shape=(size,self.output_functional.range.dim))
+            result = []
             for index in range(size):
                 output_partial_dmu = self.output_functional.d_mu(parameter, index).apply(
                     solution, mu=mu).to_numpy()[0]
                 U_d_mu = U_d_mus[parameter][index]
-                array[index] = output_partial_dmu + self.output_functional.jacobian(
-                    solution, mu).apply(U_d_mu, mu).to_numpy()[0]
+                result.append(output_partial_dmu + self.output_functional.jacobian(
+                    solution, mu).apply(U_d_mu, mu).to_numpy()[0])
+            result = np.array(result)
             if return_array:
-                gradients.extend(array)
+                gradients.extend(result)
             else:
-                gradients[parameter] = array
+                gradients[parameter] = result
         if return_array:
             return np.array(gradients)
         else:
@@ -223,7 +229,7 @@ class Model(CacheableObject, ParametricObject):
             raise ValueError('Model has no error estimator')
         return self.error_estimator.estimate_error(solution, mu, self, **kwargs)
 
-    def _compute_output_error_estimate(self, solution, mu=None, **kwargs):
+    def _compute_output_error_estimate(self, solution, mu=None, return_vector=False, **kwargs):
         """Compute an error estimate for the computed model output.
 
         This method is called by the default implementation of :meth:`compute`
@@ -244,6 +250,11 @@ class Model(CacheableObject, ParametricObject):
             Internal model state for the given |parameter values|.
         mu
             |Parameter values| for which to compute the error estimate.
+        return_vector
+            If `True`, return the output estimate as a |NumPy array|,
+            where each component corresponds to the respective component
+            of the :attr:`output_functional`.
+            Otherwise, return the euclidian norm of all components.
         kwargs
             Additional keyword arguments to customize how the error estimate is
             computed or to select additional data to be returned.
@@ -255,13 +266,16 @@ class Model(CacheableObject, ParametricObject):
         """
         if self.error_estimator is None:
             raise ValueError('Model has no error estimator')
-        return self.error_estimator.estimate_output_error(solution, mu, self, **kwargs)
+        return self.error_estimator.estimate_output_error(solution, mu, self,
+                                                          return_vector=return_vector,
+                                                          **kwargs)
 
     _compute_allowed_kwargs = frozenset()
 
     def compute(self, solution=False, output=False, solution_d_mu=False, output_d_mu=False,
                 solution_error_estimate=False, output_error_estimate=False,
-                output_d_mu_return_array=False, *, mu=None, **kwargs):
+                output_d_mu_return_array=False, output_error_estimate_return_vector=False,
+                *, mu=None, input=None, **kwargs):
         """Compute the solution of the model and associated quantities.
 
         This methods computes the output of the model it's internal state
@@ -296,27 +310,43 @@ class Model(CacheableObject, ParametricObject):
         output_error_estimate
             If `True`, return an error estimate for the computed output.
         output_d_mu_return_array
-            if `True`, return the output gradient as a |NumPy array|.
+            If `True`, return the output gradient as a |NumPy array|.
             Otherwise, return a dict of gradients for each |Parameter|.
+        output_error_estimate_return_vector
+            If `True`, return the output estimate as a |NumPy array|,
+            where each component corresponds to the respective component
+            of the :attr:`output_functional`.
+            Otherwise, return the euclidian norm of all components.
         mu
             |Parameter values| for which to compute the values.
+        input
+            The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
+            a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
+            mapping time to input, or a `str` expression whith `t` as variable that
+            can be used to instatiate an |ExpressionFunction| of this type.
+            Can be `None` if `self.dim_input == 0`.
         kwargs
-            Further keyword arguments to select further quantities that sould
+            Further keyword arguments to select further quantities that should
             be returned or to customize how the values are computed.
 
         Returns
         -------
         A dict with the computed values.
         """
-
         # make sure no unknown kwargs are passed
         assert kwargs.keys() <= self._compute_allowed_kwargs
+        assert input is not None or self.dim_input == 0
 
         # parse parameter values
         if not isinstance(mu, Mu):
             mu = self.parameters.parse(mu)
         assert self.parameters.assert_compatible(mu)
 
+        # parse input and add it to the parameter values
+        mu_input = Parameters(input=self.dim_input).parse(input)
+        input = mu_input.get_time_dependent_value('input') if mu_input.is_time_dependent('input') else mu_input['input']
+        mu = mu.with_(input=input)
+
         # log output
         # explicitly checking if logging is disabled saves some cpu cycles
         if not self.logging_disabled:
@@ -329,8 +359,8 @@ class Model(CacheableObject, ParametricObject):
                              output_error_estimate=output_error_estimate,
                              mu=mu, **kwargs)
 
-        if (solution or output or solution_error_estimate or
-                output_error_estimate or solution_d_mu or output_d_mu) \
+        if (solution or output or solution_error_estimate
+            or output_error_estimate or solution_d_mu or output_d_mu) \
            and 'solution' not in data:
             retval = self.cached_method_call(self._compute_solution, mu=mu, **kwargs)
             if isinstance(retval, dict):
@@ -382,7 +412,9 @@ class Model(CacheableObject, ParametricObject):
 
         if output_error_estimate and 'output_error_estimate' not in data:
             # TODO use caching here (requires skipping args in key generation)
-            retval = self._compute_output_error_estimate(data['solution'], mu=mu, **kwargs)
+            retval = self._compute_output_error_estimate(
+                data['solution'], mu=mu,
+                return_vector=output_error_estimate_return_vector, **kwargs)
             if isinstance(retval, dict):
                 assert 'output_error_estimate' in retval
                 data.update(retval)
@@ -391,7 +423,7 @@ class Model(CacheableObject, ParametricObject):
 
         return data
 
-    def solve(self, mu=None, return_error_estimate=False, **kwargs):
+    def solve(self, mu=None, input=None, return_error_estimate=False, **kwargs):
         """Solve the discrete problem for the |parameter values| `mu`.
 
         This method returns a |VectorArray| with a internal state
@@ -406,6 +438,12 @@ class Model(CacheableObject, ParametricObject):
         ----------
         mu
             |Parameter values| for which to solve.
+        input
+            The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
+            a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
+            mapping time to input, or a `str` expression whith `t` as variable that
+            can be used to instatiate an |ExpressionFunction| of this type.
+            Can be `None` if `self.dim_input == 0`.
         return_error_estimate
             If `True`, also return an error estimate for the computed solution.
         kwargs
@@ -421,6 +459,7 @@ class Model(CacheableObject, ParametricObject):
             solution=True,
             solution_error_estimate=return_error_estimate,
             mu=mu,
+            input=input,
             **kwargs
         )
         if return_error_estimate:
@@ -428,7 +467,8 @@ class Model(CacheableObject, ParametricObject):
         else:
             return data['solution']
 
-    def output(self, mu=None, return_error_estimate=False, **kwargs):
+    def output(self, mu=None, input=None, return_error_estimate=False,
+               return_error_estimate_vector=False, **kwargs):
         """Return the model output for given |parameter values| `mu`.
 
         This method is a convenience wrapper around :meth:`compute`.
@@ -437,8 +477,19 @@ class Model(CacheableObject, ParametricObject):
         ----------
         mu
             |Parameter values| for which to compute the output.
+        input
+            The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
+            a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
+            mapping time to input, or a `str` expression whith `t` as variable that
+            can be used to instatiate an |ExpressionFunction| of this type.
+            Can be `None` if `self.dim_input == 0`.
         return_error_estimate
             If `True`, also return an error estimate for the computed output.
+        return_error_estimate_vector
+            If `True`, return the output estimate as a |NumPy array|,
+            where each component corresponds to the respective component
+            of the :attr:`output_functional`.
+            Otherwise, return the euclidian norm of all components.
         kwargs
             Additional keyword arguments passed to :meth:`compute` that
             might affect how the solution is computed.
@@ -455,7 +506,9 @@ class Model(CacheableObject, ParametricObject):
         data = self.compute(
             output=True,
             output_error_estimate=return_error_estimate,
+            output_error_estimate_return_vector=return_error_estimate_vector,
             mu=mu,
+            input=input,
             **kwargs
         )
         if return_error_estimate:
@@ -463,7 +516,7 @@ class Model(CacheableObject, ParametricObject):
         else:
             return data['output']
 
-    def solve_d_mu(self, parameter, index, mu=None, **kwargs):
+    def solve_d_mu(self, parameter, index, mu=None, input=None, **kwargs):
         """Solve for the partial derivative of the solution w.r.t. a parameter index
 
         Parameters
@@ -474,6 +527,12 @@ class Model(CacheableObject, ParametricObject):
             parameter index for which to compute the sensitivity
         mu
             |Parameter value| for which to solve
+        input
+            The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
+            a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
+            mapping time to input, or a `str` expression whith `t` as variable that
+            can be used to instatiate an |ExpressionFunction| of this type.
+            Can be `None` if `self.dim_input == 0`.
 
         Returns
         -------
@@ -482,17 +541,24 @@ class Model(CacheableObject, ParametricObject):
         data = self.compute(
             solution_d_mu=(parameter, index),
             mu=mu,
+            input=input,
             **kwargs
         )
         return data['solution_d_mu']
 
-    def output_d_mu(self, mu=None, return_array=False, **kwargs):
-        """compute the gradient w.r.t. the parameter of the output functional
+    def output_d_mu(self, mu=None, input=None, return_array=False, **kwargs):
+        """Compute the gradient w.r.t. the parameter of the output functional.
 
         Parameters
         ----------
         mu
             |Parameter value| for which to compute the gradient
+        input
+            The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
+            a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
+            mapping time to input, or a `str` expression whith `t` as variable that
+            can be used to instatiate an |ExpressionFunction| of this type.
+            Can be `None` if `self.dim_input == 0`.
         return_array
             if `True`, return the output gradient as a |NumPy array|.
             Otherwise, return a dict of gradients for each |Parameter|.
@@ -504,12 +570,13 @@ class Model(CacheableObject, ParametricObject):
         data = self.compute(
             output_d_mu=True,
             mu=mu,
+            input=input,
             output_d_mu_return_array=return_array,
             **kwargs
         )
         return data['output_d_mu']
 
-    def estimate_error(self, mu=None, **kwargs):
+    def estimate_error(self, mu=None, input=None, **kwargs):
         """Estimate the error for the computed internal state.
 
         For given |parameter values| `mu` this method returns an
@@ -525,6 +592,12 @@ class Model(CacheableObject, ParametricObject):
         ----------
         mu
             |Parameter values| for which to estimate the error.
+        input
+            The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
+            a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
+            mapping time to input, or a `str` expression whith `t` as variable that
+            can be used to instatiate an |ExpressionFunction| of this type.
+            Can be `None` if `self.dim_input == 0`.
         kwargs
             Additional keyword arguments passed to :meth:`compute` that
             might affect how the error estimate (or the solution) is computed.
@@ -536,14 +609,11 @@ class Model(CacheableObject, ParametricObject):
         return self.compute(
             solution_error_estimate=True,
             mu=mu,
+            input=input,
             **kwargs
         )['solution_error_estimate']
 
-    @Deprecated('estimate_error')
-    def estimate(self, U, mu=None):
-        return self.estimate_error(mu)
-
-    def estimate_output_error(self, mu=None, **kwargs):
+    def estimate_output_error(self, mu=None, input=None, return_vector=False, **kwargs):
         """Estimate the error for the computed output.
 
         For given |parameter values| `mu` this method returns an
@@ -559,6 +629,17 @@ class Model(CacheableObject, ParametricObject):
         ----------
         mu
             |Parameter values| for which to estimate the error.
+        input
+            The model input. Either a |NumPy array| of shape `(self.dim_input,)`,
+            a |Function| with `dim_domain == 1` and `shape_range == (self.dim_input,)`
+            mapping time to input, or a `str` expression whith `t` as variable that
+            can be used to instatiate an |ExpressionFunction| of this type.
+            Can be `None` if `self.dim_input == 0`.
+        return_vector
+            If `True`, return the output estimate as a |NumPy array|,
+            where each component corresponds to the respective component
+            of the :attr:`output_functional`.
+            Otherwise, return the euclidian norm of all components.
         kwargs
             Additional keyword arguments passed to :meth:`compute` that
             might affect how the error estimate (or the output) is computed.
@@ -569,7 +650,9 @@ class Model(CacheableObject, ParametricObject):
         """
         return self.compute(
             output_error_estimate=True,
+            output_error_estimate_return_vector=return_vector,
             mu=mu,
+            input=input,
             **kwargs
         )['output_error_estimate']
 
@@ -586,6 +669,6 @@ class Model(CacheableObject, ParametricObject):
             See the docstring of `self.visualizer.visualize`.
         """
         if getattr(self, 'visualizer') is not None:
-            return self.visualizer.visualize(U, self, **kwargs)
+            return self.visualizer.visualize(U, **kwargs)
         else:
             raise NotImplementedError('Model has no visualizer.')
