@@ -1,6 +1,6 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 """Reductors based on H2-norm."""
 
@@ -11,15 +11,16 @@ import scipy.linalg as spla
 
 from pymor.algorithms.gram_schmidt import gram_schmidt, gram_schmidt_biorth
 from pymor.algorithms.krylov import tangential_rational_krylov
+from pymor.algorithms.riccati import solve_ricc_dense
 from pymor.algorithms.sylvester import solve_sylv_schur
 from pymor.algorithms.to_matrix import to_matrix
 from pymor.core.base import BasicObject
-from pymor.models.iosys import InputOutputModel, LTIModel
+from pymor.models.iosys import LTIModel, _lti_to_poles_b_c, _poles_b_c_to_lti
+from pymor.models.transfer_function import TransferFunction
 from pymor.operators.constructions import IdentityOperator
 from pymor.parameters.base import Mu
 from pymor.reductors.basic import LTIPGReductor
 from pymor.reductors.interpolation import LTIBHIReductor, TFBHIReductor
-from pymor.vectorarrays.numpy import NumpyVectorSpace
 
 
 class GenericIRKAReductor(BasicObject):
@@ -64,7 +65,7 @@ class GenericIRKAReductor(BasicObject):
         elif isinstance(rom0_params, np.ndarray):
             assert rom0_params.ndim == 1
         elif isinstance(rom0_params, dict):
-            assert ('sigma', 'b', 'c') in rom0_params
+            assert {'sigma', 'b', 'c'} <= rom0_params.keys()
             assert isinstance(rom0_params['sigma'], np.ndarray)
             assert isinstance(rom0_params['b'], np.ndarray)
             assert isinstance(rom0_params['c'], np.ndarray)
@@ -76,7 +77,7 @@ class GenericIRKAReductor(BasicObject):
         elif isinstance(rom0_params, LTIModel):
             assert rom0_params.order > 0
             if hasattr(self.fom, 'order'):  # self.fom can be a TransferFunction
-                assert rom0_params < self.fom.order
+                assert rom0_params.order < self.fom.order
             assert rom0_params.dim_input == self.fom.dim_input
             assert rom0_params.dim_output == self.fom.dim_output
         else:
@@ -191,6 +192,7 @@ class IRKAReductor(GenericIRKAReductor):
     mu
         |Parameter values|.
     """
+
     def __init__(self, fom, mu=None):
         assert isinstance(fom, LTIModel)
         super().__init__(fom, mu=mu)
@@ -200,7 +202,7 @@ class IRKAReductor(GenericIRKAReductor):
                compute_errors=False):
         r"""Reduce using IRKA.
 
-        See [GAB08]_ (Algorithm 4.1) and [ABG10]_ (Algorithm 1).
+        See :cite:`GAB08` (Algorithm 4.1) and :cite:`ABG10` (Algorithm 1).
 
         Parameters
         ----------
@@ -258,7 +260,7 @@ class IRKAReductor(GenericIRKAReductor):
         rom
             Reduced |LTIModel| model.
         """
-        if not self.fom.cont_time:
+        if self.fom.sampling_time > 0:
             raise NotImplementedError
 
         self._clear_lists()
@@ -304,6 +306,7 @@ class OneSidedIRKAReductor(GenericIRKAReductor):
     mu
         |Parameter values|.
     """
+
     def __init__(self, fom, version, mu=None):
         assert isinstance(fom, LTIModel)
         assert version in ('V', 'W')
@@ -369,7 +372,7 @@ class OneSidedIRKAReductor(GenericIRKAReductor):
         rom
             Reduced |LTIModel| model.
         """
-        if not self.fom.cont_time:
+        if self.fom.sampling_time > 0:
             raise NotImplementedError
 
         self._clear_lists()
@@ -431,6 +434,7 @@ class TSIAReductor(GenericIRKAReductor):
     mu
         |Parameter values|.
     """
+
     def __init__(self, fom, mu=None):
         assert isinstance(fom, LTIModel)
         super().__init__(fom, mu=mu)
@@ -439,7 +443,7 @@ class TSIAReductor(GenericIRKAReductor):
                conv_crit='sigma', compute_errors=False):
         r"""Reduce using TSIA.
 
-        See [XZ11]_ (Algorithm 1) and [BKS11]_.
+        See :cite:`XZ11` (Algorithm 1) and :cite:`BKS11`.
 
         In exact arithmetic, TSIA is equivalent to IRKA (under some
         assumptions on the poles of the reduced model). The main
@@ -498,7 +502,7 @@ class TSIAReductor(GenericIRKAReductor):
         rom
             Reduced |LTIModel|.
         """
-        if not self.fom.cont_time:
+        if self.fom.sampling_time > 0:
             raise NotImplementedError
 
         self._clear_lists()
@@ -547,17 +551,22 @@ class TSIAReductor(GenericIRKAReductor):
 class TFIRKAReductor(GenericIRKAReductor):
     """Realization-independent IRKA reductor.
 
-    See [BG12]_.
+    See :cite:`BG12`.
 
     Parameters
     ----------
     fom
-        The full-order |Model| with `eval_tf` and `eval_dtf` methods.
+        |TransferFunction| or |Model| with a `transfer_function` attribute,
+        with `eval_tf` and `eval_dtf` methods that should be defined at least
+        over the open right half of the complex plane.
     mu
         |Parameter values|.
     """
+
     def __init__(self, fom, mu=None):
-        assert isinstance(fom, InputOutputModel)
+        assert isinstance(fom, TransferFunction) or hasattr(fom, 'transfer_function')
+        if not isinstance(fom, TransferFunction):
+            fom = fom.transfer_function
         super().__init__(fom, mu=mu)
 
     def reduce(self, rom0_params, tol=1e-4, maxit=100, num_prev=1,
@@ -611,7 +620,7 @@ class TFIRKAReductor(GenericIRKAReductor):
         rom
             Reduced |LTIModel| model.
         """
-        if not self.fom.cont_time:
+        if self.fom.sampling_time > 0:
             raise NotImplementedError
 
         self._clear_lists()
@@ -643,75 +652,178 @@ class TFIRKAReductor(GenericIRKAReductor):
         )
 
 
-def _lti_to_poles_b_c(rom):
-    """Compute poles and residues.
+class GapIRKAReductor(GenericIRKAReductor):
+    """Gap-IRKA reductor.
 
     Parameters
     ----------
-    rom
-        Reduced |LTIModel| (consisting of |NumpyMatrixOperators|).
-
-    Returns
-    -------
-    poles
-        1D |NumPy array| of poles.
-    b
-        |NumPy array| of shape `(rom.order, rom.dim_input)`.
-    c
-        |NumPy array| of shape `(rom.order, rom.dim_output)`.
+    fom
+        The full-order |LTIModel| to reduce.
+    mu
+        |Parameter|.
     """
-    A = to_matrix(rom.A, format='dense')
-    B = to_matrix(rom.B, format='dense')
-    C = to_matrix(rom.C, format='dense')
-    if isinstance(rom.E, IdentityOperator):
-        poles, X = spla.eig(A)
-        EX = X
-    else:
-        E = to_matrix(rom.E, format='dense')
-        poles, X = spla.eig(A, E)
-        EX = E @ X
-    b = spla.solve(EX, B)
-    c = (C @ X).T
-    return poles, b, c
 
+    def __init__(self, fom, mu=None, solver_options=None):
+        assert isinstance(fom, LTIModel)
+        super().__init__(fom, mu=mu)
+        self.solver_options = solver_options
 
-def _poles_b_c_to_lti(poles, b, c):
-    r"""Create an |LTIModel| from poles and residue rank-1 factors.
+    def reduce(self, rom0_params, tol=1e-4, maxit=100, num_prev=1, conv_crit='sigma',
+               projection='orth'):
+        r"""Reduce using gap-IRKA.
 
-    Returns an |LTIModel| with real matrices such that its transfer
-    function is
+        See :cite:`BBG19` Algorithm 1.
 
-    .. math::
-        \sum_{i = 1}^r \frac{c_i b_i^T}{s - \lambda_i}
+        Parameters
+        ----------
+        rom0_params
+            Can be:
 
-    where :math:`\lambda_i, b_i, c_i` are the poles and residue rank-1
-    factors.
+            - order of the reduced model (a positive integer),
+            - initial interpolation points (a 1D |NumPy array|),
+            - dict with `'sigma'`, `'b'`, `'c'` as keys mapping to
+              initial interpolation points (a 1D |NumPy array|), right
+              tangential directions (|VectorArray| from
+              `fom.input_space`), and left tangential directions
+              (|VectorArray| from `fom.output_space`), all of the same
+              length (the order of the reduced model),
+            - initial reduced-order model (|LTIModel|).
 
-    Parameters
-    ----------
-    poles
-        Sequence of poles.
-    b
-        |NumPy array| of shape `(rom.order, rom.dim_input)`.
-    c
-        |NumPy array| of shape `(rom.order, rom.dim_output)`.
+            If the order of reduced model is given, initial
+            interpolation data is generated randomly.
+        tol
+            Tolerance for the convergence criterion.
+        maxit
+            Maximum number of iterations.
+        num_prev
+            Number of previous iterations to compare the current
+            iteration to. A larger number can avoid occasional cyclic
+            behavior.
+        conv_crit
+            Convergence criterion:
 
-    Returns
-    -------
-    |LTIModel|.
-    """
-    A, B, C = [], [], []
-    for i, pole in enumerate(poles):
-        if pole.imag == 0:
-            A.append(pole.real)
-            B.append(b[i].real)
-            C.append(c[i].real[:, np.newaxis])
-        elif pole.imag > 0:
-            A.append([[pole.real, pole.imag],
-                      [-pole.imag, pole.real]])
-            B.append(np.vstack([2 * b[i].real, -2 * b[i].imag]))
-            C.append(np.hstack([c[i].real[:, np.newaxis], c[i].imag[:, np.newaxis]]))
-    A = spla.block_diag(*A)
-    B = np.vstack(B)
-    C = np.hstack(C)
-    return LTIModel.from_matrices(A, B, C)
+            - `'sigma'`: relative change in interpolation points
+            - `'htwogap'`: :math:`\mathcal{H}_2-gap` distance of
+              reduced-order models divided by :math:`\mathcal{L}_2`
+              norm of new reduced-order model
+            - `'ltwo'`: relative :math:`\mathcal{L}_2` distance of
+              reduced-order models
+        projection
+            Projection method:
+
+            - `'orth'`: projection matrix is orthogonalized with respect
+              to the Euclidean inner product,
+            - `'biorth'`: projection matrix is orthogonalized with
+              respect to the E product.
+
+        Returns
+        -------
+        rom
+            Reduced |LTIModel| model.
+        """
+        if self.fom.sampling_time > 0:
+            raise NotImplementedError
+
+        self._clear_lists()
+        sigma, b, c = self._rom0_params_to_sigma_b_c(rom0_params)
+        self._store_sigma_b_c(sigma, b, c)
+        assert projection in ('orth', 'biorth')
+
+        self.logger.info('Starting gap IRKA')
+        self._conv_data = (num_prev + 1) * [None]
+        if conv_crit == 'sigma':
+            self._conv_data[0] = sigma
+        for it in range(maxit):
+            self._pg_reductor = LTIBHIReductor(self.fom, mu=self.mu)
+            rom = self._pg_reductor.reduce(sigma, b, c, projection)
+            sigma, b, c, gap_rom = self._unstable_rom_to_sigma_b_c(rom)
+            self._store_sigma_b_c(sigma, b, c)
+            self._update_conv_data(sigma, gap_rom, rom, conv_crit)
+            self._compute_conv_crit(rom, conv_crit, it)
+            if self.conv_crit[-1] < tol:
+                break
+
+        return rom
+
+    def _rom0_params_to_sigma_b_c(self, rom0_params):
+        self.logger.info('Generating initial interpolation data')
+        self._check_rom0_params(rom0_params)
+        if isinstance(rom0_params, Integral):
+            sigma, b, c = self._order_to_sigma_b_c(rom0_params)
+        elif isinstance(rom0_params, np.ndarray):
+            sigma = rom0_params
+            _, b, c = self._order_to_sigma_b_c(len(rom0_params))
+        elif isinstance(rom0_params, dict):
+            sigma = rom0_params['sigma']
+            b = rom0_params['b']
+            c = rom0_params['c']
+        else:
+            sigma, b, c, _ = self._unstable_rom_to_sigma_b_c(rom0_params)
+        return sigma, b, c
+
+    def _unstable_rom_to_sigma_b_c(self, rom):
+        poles, b, c, gap_rom = self._unstable_lti_to_poles_b_c(rom)
+        return -poles, b, c, gap_rom
+
+    def _unstable_lti_to_poles_b_c(self, rom):
+        A = to_matrix(rom.A, format='dense')
+        B = to_matrix(rom.B, format='dense')
+        C = to_matrix(rom.C, format='dense')
+
+        options = self.solver_options
+
+        if isinstance(rom.E, IdentityOperator):
+            P = solve_ricc_dense(A, None, B, C, options=options)
+            F = P @ C.T
+            AF = A - F @ C
+            poles, X = spla.eig(AF)
+            EX = X
+        else:
+            E = to_matrix(rom.E, format='dense')
+            P = solve_ricc_dense(A, E, B, C, options=options)
+            F = E @ P @ C.T
+            AF = A - F @ C
+            poles, X = spla.eig(AF, E)
+            EX = E @ X
+        b = spla.solve(EX, B)
+        c = (C @ X).T
+        mFB = np.concatenate((-F, B), axis=1)
+        gap_rom = LTIModel.from_matrices(AF, mFB, C, E=None if isinstance(rom.E, IdentityOperator) else E)
+        return poles, b, c, gap_rom
+
+    def _update_conv_data(self, sigma, gap_rom, rom, conv_crit):
+        del self._conv_data[-1]
+        if conv_crit == 'sigma':
+            self._conv_data.insert(0, sigma)
+        elif conv_crit == 'htwogap':
+            self._conv_data.insert(0, gap_rom)
+        elif conv_crit == 'ltwo':
+            self._conv_data.insert(0, rom)
+
+    def _compute_conv_crit(self, rom, conv_crit, it):
+        if conv_crit == 'sigma':
+            sigma = self._conv_data[0]
+            dist = min(spla.norm((sigma_old - sigma) / sigma_old, ord=np.inf)
+                       for sigma_old in self._conv_data[1:]
+                       if sigma_old is not None)
+        elif conv_crit == 'htwogap':
+            gap_rom = self._conv_data[0]
+            A = to_matrix(rom.A, format='dense')
+            C = to_matrix(rom.C, format='dense')
+            F = -to_matrix(gap_rom.B, format='dense')[:, :C.shape[0]]
+            D = np.eye(rom.C.range.dim)
+            E = to_matrix(rom.E, format='dense') if isinstance(rom.E, IdentityOperator) else None
+
+            mi = LTIModel.from_matrices(A, F, C, D, E)
+            dist = min((gap_rom_old - gap_rom).h2_norm() * mi.linf_norm() / rom.l2_norm()
+                       if gap_rom_old is not None
+                       else np.inf
+                       for gap_rom_old in self._conv_data[1:])
+        elif conv_crit == 'ltwo':
+            dist = min((rom_old - rom).l2_norm() / rom.l2_norm()
+                       if rom_old is not None
+                       else np.inf
+                       for rom_old in self._conv_data[1:])
+
+        self.conv_crit.append(dist)
+        self.logger.info(f'Convergence criterion in iteration {it + 1}: {dist:e}')
