@@ -1,5 +1,5 @@
 # This file is part of the pyMOR project (https://www.pymor.org).
-# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# Copyright pyMOR developers and contributors. All rights reserved.
 # License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 from functools import reduce
@@ -7,7 +7,161 @@ from numbers import Number
 import numpy as np
 
 from pymor.core.base import classinstancemethod
-from pymor.vectorarrays.interface import VectorArray, VectorSpace
+from pymor.tools.deprecated import Deprecated
+from pymor.vectorarrays.interface import VectorArray, VectorArrayImpl, VectorSpace
+
+
+class BlockVectorArrayImpl(VectorArrayImpl):
+
+    def __init__(self, blocks, space):
+        self._blocks = tuple(blocks)
+        self.space = space
+        assert self._blocks_are_valid(False)
+
+    def __len__(self):
+        try:
+            return len(self._blocks[0])
+        except IndexError:
+            return 0
+
+    def to_numpy(self, ensure_copy, ind):
+        assert self._blocks_are_valid()
+        if len(self._blocks):
+            # hstack will error out with empty input list
+            return np.hstack([_indexed(block, ind).to_numpy(False) for block in self._blocks])
+        else:
+            return np.empty((0, 0))
+
+    def real(self, ind):
+        assert self._blocks_are_valid()
+        return type(self)([_indexed(block, ind).real for block in self._blocks], self.space)
+
+    def imag(self, ind):
+        assert self._blocks_are_valid()
+        return type(self)([_indexed(block, ind).imag for block in self._blocks], self.space)
+
+    def conj(self, ind):
+        assert self._blocks_are_valid()
+        return type(self)([_indexed(block, ind).conj() for block in self._blocks], self.space)
+
+    def delete(self, ind):
+        assert self._blocks_are_valid()
+        ind = slice(None) if ind is None else ind
+        for block in self._blocks:
+            del block[ind]
+
+    def append(self, other, remove_from_other, oind):
+        assert self._blocks_are_valid()
+        assert other._blocks_are_valid()
+        for block, other_block in zip(self._blocks, other._blocks):
+            block.append(_indexed(other_block, oind), remove_from_other)
+
+    def copy(self, deep, ind):
+        assert self._blocks_are_valid()
+        return type(self)([_indexed(block, ind).copy(deep) for block in self._blocks], self.space)
+
+    def scal(self, alpha, ind):
+        assert self._blocks_are_valid()
+        for block in self._blocks:
+            _indexed(block, ind).scal(alpha)
+
+    def scal_copy(self, alpha, ind):
+        assert self._blocks_are_valid()
+        if isinstance(alpha, Number):
+            if alpha == -1:
+                return type(self)([-_indexed(block, ind) for block in self._blocks], self.space)
+        return super().scal_copy(alpha, ind)
+
+    def axpy(self, alpha, x, ind, xind):
+        assert self._blocks_are_valid()
+        assert x._blocks_are_valid()
+        for block, x_block in zip(self._blocks, x._blocks):
+            _indexed(block, ind).axpy(alpha, _indexed(x_block, xind))
+
+    def axpy_copy(self, alpha, x, ind, xind):
+        assert self._blocks_are_valid()
+        assert x._blocks_are_valid()
+        if isinstance(alpha, Number):
+            if alpha == 1:
+                return type(self)([_indexed(block, ind) + _indexed(x_block, xind)
+                                   for block, x_block in zip(self._blocks, x._blocks)],
+                                  self.space)
+            elif alpha == -1:
+                return type(self)([_indexed(block, ind) - _indexed(x_block, xind)
+                                   for block, x_block in zip(self._blocks, x._blocks)],
+                                  self.space)
+        return super().axpy_copy(alpha, x, ind, xind)
+
+    def inner(self, other, ind, oind):
+        assert self._blocks_are_valid()
+        assert other._blocks_are_valid()
+        prods = [_indexed(block, ind).inner(_indexed(other_block, oind))
+                 for block, other_block in zip(self._blocks, other._blocks)]
+        assert all([prod.shape == prods[0].shape for prod in prods])
+        common_dtype = reduce(np.promote_types, (prod.dtype for prod in prods))
+        ret = np.zeros(prods[0].shape, dtype=common_dtype)
+        for prod in prods:
+            ret += prod
+        return ret
+
+    def pairwise_inner(self, other, ind, oind):
+        assert self._blocks_are_valid()
+        assert other._blocks_are_valid()
+        prods = [_indexed(block, ind).pairwise_inner(_indexed(other_block, oind))
+                 for block, other_block in zip(self._blocks, other._blocks)]
+        assert all([prod.shape == prods[0].shape for prod in prods])
+        common_dtype = reduce(np.promote_types, (prod.dtype for prod in prods))
+        ret = np.zeros(prods[0].shape, dtype=common_dtype)
+        for prod in prods:
+            ret += prod
+        return ret
+
+    def lincomb(self, coefficients, ind):
+        assert self._blocks_are_valid()
+        lincombs = [_indexed(block, ind).lincomb(coefficients) for block in self._blocks]
+        return type(self)(lincombs, self.space)
+
+    def norm2(self, ind):
+        assert self._blocks_are_valid()
+        return np.sum(np.array([_indexed(block, ind).norm2() for block in self._blocks]), axis=0)
+
+    def dofs(self, dof_indices, ind):
+        assert self._blocks_are_valid()
+        if not len(dof_indices):
+            return np.zeros((self.len_ind(ind), 0))
+
+        self._compute_bins()
+        block_inds = np.digitize(dof_indices, self._bins) - 1
+        dof_indices = dof_indices - self._bins[block_inds]
+        block_inds = self._bin_map[block_inds]
+        blocks = [_indexed(b, ind) for b in self._blocks]
+        return np.array([blocks[bi].dofs([ci])[:, 0]
+                         for bi, ci in zip(block_inds, dof_indices)]).T
+
+    def amax(self, ind):
+        assert self._blocks_are_valid()
+        self._compute_bins()
+        blocks = [_indexed(b, ind) for b in self._blocks]
+        inds, vals = zip(*(blocks[bi].amax() for bi in self._bin_map))
+        inds, vals = np.array(inds), np.array(vals)
+        inds += self._bins[:-1][..., np.newaxis]
+        block_inds = np.argmax(vals, axis=0)
+        ar = np.arange(inds.shape[1])
+        return inds[block_inds, ar], vals[block_inds, ar]
+
+    def _blocks_are_valid(self, after_creation=True):
+        assert all([len(block) == len(self._blocks[0]) for block in self._blocks]), \
+            (f'All blocks in a BlockVectorArray must be of the same length '
+             f'(lengths: {[len(block) for block in self._blocks]})'
+             + ('Likely, an array has been modified that has been accessed via BlockVectorArray.blocks'
+                if after_creation else ''))
+        return True
+
+    def _compute_bins(self):
+        if not hasattr(self, '_bins'):
+            dims = np.array([subspace.dim for subspace in self.space.subspaces])
+            self._bin_map = bin_map = np.where(dims > 0)[0]
+            self._bins = np.cumsum(np.hstack(([0], dims[bin_map])))
 
 
 class BlockVectorArray(VectorArray):
@@ -21,154 +175,35 @@ class BlockVectorArray(VectorArray):
     :class:`~pymor.operators.block.BlockOperator`.
     """
 
-    def __init__(self, blocks, space):
-        self._blocks = tuple(blocks)
-        self.space = space
-        assert self._blocks_are_valid()
-
-    def to_numpy(self, ensure_copy=False):
-        if len(self._blocks):
-            # hstack will error out with empty input list
-            return np.hstack([block.to_numpy() for block in self._blocks])
-        else:
-            return np.empty((0, 0))
-
-    @property
-    def real(self):
-        return BlockVectorArray([block.real for block in self._blocks], self.space)
-
-    @property
-    def imag(self):
-        return BlockVectorArray([block.imag for block in self._blocks], self.space)
-
-    def conj(self):
-        return BlockVectorArray([block.conj() for block in self._blocks], self.space)
+    impl_type = BlockVectorArrayImpl
 
+    @Deprecated('BlockVectorArray.blocks')
     def block(self, ind):
         """Return a copy of a single block or a sequence of blocks."""
-        assert self._blocks_are_valid()
         if isinstance(ind, (tuple, list)):
             assert all(isinstance(ii, Number) for ii in ind)
-            return tuple(self._blocks[ii].copy() for ii in ind)
+            return tuple(_indexed(self.impl._blocks[ii], self.ind).copy() for ii in ind)
         else:
             assert isinstance(ind, Number)
-            return self._blocks[ind].copy()
+            return _indexed(self.impl._blocks[ind], self.ind).copy()
 
     @property
+    @Deprecated('len(BlockVectorArray.blocks)')
     def num_blocks(self):
-        return len(self._blocks)
+        return len(self.space.subspaces)
 
-    def __len__(self):
-        try:
-            return len(self._blocks[0])
-        except IndexError:
-            return 0
-
-    def __getitem__(self, ind):
-        if isinstance(ind, Number) and (ind >= len(self) or ind < -len(self)):
-            raise IndexError('VectorArray index out of range')
-        assert self.check_ind(ind)
-        return BlockVectorArrayView(self, ind)
-
-    def __delitem__(self, ind):
-        assert self.check_ind(ind)
-        for block in self._blocks:
-            del block[ind]
-
-    def append(self, other, remove_from_other=False):
-        assert self._blocks_are_valid()
-        assert other in self.space
-        for block, other_block in zip(self._blocks, other._blocks):
-            block.append(other_block, remove_from_other=remove_from_other)
-
-    def copy(self, deep=False):
-        return BlockVectorArray([block.copy(deep) for block in self._blocks], self.space)
-
-    def scal(self, alpha):
-        for block in self._blocks:
-            block.scal(alpha)
-
-    def axpy(self, alpha, x):
-        assert x in self.space
-        assert isinstance(alpha, Number) \
-            or isinstance(alpha, np.ndarray) and alpha.shape == (len(self),)
-        if len(x) > 0:
-            for block, x_block in zip(self._blocks, x._blocks):
-                block.axpy(alpha, x_block)
+    @property
+    def blocks(self):
+        if self.ind is None:
+            return self.impl._blocks
         else:
-            assert len(self) == 0
-
-    def inner(self, other, product=None):
-        assert other in self.space
-        if product is not None:
-            return product.apply2(self, other)
-
-        prods = [block.inner(other_block) for block, other_block in zip(self._blocks, other._blocks)]
-        assert all([prod.shape == prods[0].shape for prod in prods])
-        common_dtype = reduce(np.promote_types, (prod.dtype for prod in prods))
-        ret = np.zeros(prods[0].shape, dtype=common_dtype)
-        for prod in prods:
-            ret += prod
-        return ret
-
-    def pairwise_inner(self, other, product=None):
-        assert other in self.space
-        if product is not None:
-            return product.pairwise_apply2(self, other)
-
-        prods = [block.pairwise_inner(other_block)
-                 for block, other_block in zip(self._blocks, other._blocks)]
-        assert all([prod.shape == prods[0].shape for prod in prods])
-        common_dtype = reduce(np.promote_types, (prod.dtype for prod in prods))
-        ret = np.zeros(prods[0].shape, dtype=common_dtype)
-        for prod in prods:
-            ret += prod
-        return ret
-
-    def lincomb(self, coefficients):
-        lincombs = [block.lincomb(coefficients) for block in self._blocks]
-        return BlockVectorArray(lincombs, self.space)
-
-    def _norm(self):
-        return np.sqrt(self.norm2())
-
-    def _norm2(self):
-        return np.sum(np.array([block.norm2() for block in self._blocks]), axis=0)
-
-    def sup_norm(self):
-        return np.max(np.array([block.sup_norm() for block in self._blocks]), axis=0)
-
-    def dofs(self, dof_indices):
-        dof_indices = np.array(dof_indices)
-        if not len(dof_indices):
-            return np.zeros((len(self), 0))
-
-        self._compute_bins()
-        block_inds = np.digitize(dof_indices, self._bins) - 1
-        dof_indices -= self._bins[block_inds]
-        block_inds = self._bin_map[block_inds]
-        blocks = self._blocks
-        return np.array([blocks[bi].dofs([ci])[:, 0]
-                         for bi, ci in zip(block_inds, dof_indices)]).T
-
-    def amax(self):
-        self._compute_bins()
-        blocks = self._blocks
-        inds, vals = zip(*(blocks[bi].amax() for bi in self._bin_map))
-        inds, vals = np.array(inds), np.array(vals)
-        inds += self._bins[:-1][..., np.newaxis]
-        block_inds = np.argmax(vals, axis=0)
-        ar = np.arange(inds.shape[1])
-        return inds[block_inds, ar], vals[block_inds, ar]
-
-    def _blocks_are_valid(self):
-        return all([len(block) == len(self._blocks[0]) for block in self._blocks])
-
-    def _compute_bins(self):
-        if not hasattr(self, '_bins'):
-            dims = np.array([subspace.dim for subspace in self.space.subspaces])
-            self._bin_map = bin_map = np.where(dims > 0)[0]
-            self._bins = np.cumsum(np.hstack(([0], dims[bin_map])))
+            try:
+                return self._blocks
+            except AttributeError:
+                pass
+            ind = self.ind
+            self._blocks = tuple(b[ind] for b in self.impl._blocks)
+            return self._blocks
 
 
 class BlockVectorSpace(VectorSpace):
@@ -207,7 +242,10 @@ class BlockVectorSpace(VectorSpace):
         # these asserts make sure we also trigger if the subspace list is empty
         assert count >= 0
         assert reserve >= 0
-        return BlockVectorArray([subspace.zeros(count=count, reserve=reserve) for subspace in self.subspaces], self)
+        return BlockVectorArray(
+            self,
+            BlockVectorArrayImpl([subspace.zeros(count=count, reserve=reserve) for subspace in self.subspaces], self)
+        )
 
     @classinstancemethod
     def make_array(cls, obj):
@@ -219,7 +257,7 @@ class BlockVectorSpace(VectorSpace):
         """:noindex:"""
         assert len(obj) == len(self.subspaces)
         assert all(block in subspace for block, subspace in zip(obj, self.subspaces))
-        return BlockVectorArray(obj, self)
+        return BlockVectorArray(self, BlockVectorArrayImpl(obj, self))
 
     def make_block_diagonal_array(self, obj):
         assert len(obj) == len(self.subspaces)
@@ -233,14 +271,13 @@ class BlockVectorSpace(VectorSpace):
         if data.ndim == 1:
             data = data.reshape(1, -1)
         data_ind = np.cumsum([0] + [subspace.dim for subspace in self.subspaces])
-        return BlockVectorArray([subspace.from_numpy(data[:, data_ind[i]:data_ind[i + 1]], ensure_copy=ensure_copy)
-                                 for i, subspace in enumerate(self.subspaces)], self)
+        return BlockVectorArray(
+            self,
+            BlockVectorArrayImpl([subspace.from_numpy(data[:, data_ind[i]:data_ind[i + 1]], ensure_copy=ensure_copy)
+                                  for i, subspace in enumerate(self.subspaces)],
+                                 self)
+        )
 
 
-class BlockVectorArrayView(BlockVectorArray):
-
-    is_view = True
-
-    def __init__(self, base, ind):
-        self._blocks = tuple(block[ind] for block in base._blocks)
-        self.space = base.space
+def _indexed(block, ind):
+    return block if ind is None else block[ind]
