@@ -1,5 +1,5 @@
 # This file is part of the pyMOR project (https://www.pymor.org).
-# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# Copyright pyMOR developers and contributors. All rights reserved.
 # License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 from pymor.operators.interface import Operator
@@ -89,4 +89,4 @@ class WrappedDiffusionOperator(Operator):
             self.op.apply(u._impl, v)
             return v
 
-        return self.range.make_array([apply_one_vector(u) for u in U._list])
+        return self.range.make_array([apply_one_vector(u) for u in U.vectors])
