@@ -1,12 +1,13 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import numpy as np
 from numpy.polynomial.polynomial import Polynomial
 import pytest
 
 from pymor.core.config import config
+from pymor.operators.constructions import IdentityOperator
 from pymor.operators.interface import Operator
 from pymor.operators.list import NumpyListVectorArrayMatrixOperator
 from pymor.operators.numpy import NumpyMatrixOperator
@@ -340,7 +341,7 @@ thermalblock_fixedparam_operator_with_arrays_and_products_generators = \
     [lambda args=args: thermalblock_fixedparam_factory(*args) for args in thermalblock_factory_arguments]
 
 
-num_misc_operators = 12
+num_misc_operators = 14
 
 
 def misc_operator_with_arrays_and_products_factory(n):
@@ -442,6 +443,18 @@ def misc_operator_with_arrays_and_products_factory(n):
         U = op.source.make_array([U0, U1])
         V = V0
         return op, mu, U, V, sp, rp
+    elif n == 12:
+        from pymor.operators.constructions import NumpyConversionOperator
+        from pymor.vectorarrays.block import BlockVectorSpace
+        space = BlockVectorSpace([NumpyVectorSpace(1), NumpyVectorSpace(2)])
+        op = NumpyConversionOperator(space)
+        return op, None, op.source.random(), op.range.random(), IdentityOperator(op.source), IdentityOperator(op.range)
+    elif n == 13:
+        from pymor.operators.numpy import NumpyHankelOperator
+        s, p, m = 4, 2, 3
+        mp = np.random.rand(s, p, m)
+        op = NumpyHankelOperator(mp)
+        return op, None, op.source.random(), op.range.random(), IdentityOperator(op.source), IdentityOperator(op.range)
     else:
         assert False
 
@@ -489,6 +502,24 @@ unpicklable_misc_operator_with_arrays_and_products_generators = \
 
 
 if config.HAVE_FENICS:
+    def fenics_matrix_operator_factory():
+        import dolfin as df
+        from pymor.bindings.fenics import FenicsMatrixOperator
+
+        mesh = df.UnitSquareMesh(10, 10)
+        V = df.FunctionSpace(mesh, "CG", 2)
+
+        u = df.TrialFunction(V)
+        v = df.TestFunction(V)
+        c = df.Constant([1, 1])
+
+        op = FenicsMatrixOperator(
+            df.assemble(u * v * df.dx + df.inner(c, df.grad(u)) * v * df.dx),
+            V, V)
+
+        prod = FenicsMatrixOperator(df.assemble(u*v*df.dx), V, V)
+        return op, None, op.source.random(), op.range.random(), prod, prod
+
     def fenics_nonlinear_operator_factory():
         import dolfin as df
         from pymor.bindings.fenics import FenicsVectorSpace, FenicsOperator, FenicsMatrixOperator
@@ -520,11 +551,15 @@ if config.HAVE_FENICS:
         prod = FenicsMatrixOperator(df.assemble(u*v*df.dx), V, V)
         return op, op.parameters.parse(42), op.source.random(), op.range.random(), prod, prod
 
-    fenics_with_arrays_and_products_generators = [lambda: fenics_nonlinear_operator_factory()]
-    fenics_with_arrays_generators = [lambda: fenics_nonlinear_operator_factory()[:4]]
-
+    fenics_with_arrays_and_products_generators = [
+        lambda: fenics_matrix_operator_factory(),
+        lambda: fenics_nonlinear_operator_factory(),
+    ]
+    fenics_with_arrays_generators = [
+        lambda: fenics_matrix_operator_factory()[:4],
+        lambda: fenics_nonlinear_operator_factory()[:4],
+    ]
 else:
-
     fenics_with_arrays_and_products_generators = []
     fenics_with_arrays_generators = []
 
@@ -606,3 +641,8 @@ def operator(request):
 ))
 def picklable_operator(request):
     return request.param()
+
+
+@pytest.fixture
+def loadable_matrices(shared_datadir):
+    return (shared_datadir / "matrices").glob('*')
