@@ -1,12 +1,13 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import numpy as np
 import scipy.linalg as spla
 import scipy.sparse as sps
 
 from pymor.algorithms.to_matrix import to_matrix
+from pymor.core.config import config
 from pymor.operators.block import BlockOperator, BlockDiagonalOperator
 from pymor.operators.constructions import (AdjointOperator, ComponentProjectionOperator, IdentityOperator,
                                            LowRankOperator, LowRankUpdatedOperator, VectorArrayOperator, ZeroOperator)
@@ -255,3 +256,25 @@ def test_to_matrix_ZeroOperator():
 
     Zop = ZeroOperator(NumpyVectorSpace(n), NumpyVectorSpace(m))
     assert_type_and_allclose(Z, Zop, 'sparse')
+
+
+if config.HAVE_DUNEGDT:
+    from dune.xt.la import IstlSparseMatrix, SparsityPatternDefault
+    from pymor.bindings.dunegdt import DuneXTMatrixOperator
+
+    def test_to_matrix_DuneXTMatrixOperator():
+        np.random.seed(0)
+        A = np.random.randn(2, 2)
+
+        pattern = SparsityPatternDefault(2)
+        for ii in range(2):
+            for jj in range(2):
+                pattern.insert(ii, jj)
+        pattern.sort()
+        mat = IstlSparseMatrix(2, 2, pattern)
+        for ii in range(2):
+            for jj in range(2):
+                mat.set_entry(ii, jj, A[ii][jj])
+        Aop = DuneXTMatrixOperator(mat)
+
+        assert_type_and_allclose(A, Aop, 'sparse')
