@@ -1,22 +1,23 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
-from itertools import product, chain
 from numbers import Number
 
 import pytest
 import numpy as np
-from hypothesis import given, assume, settings, HealthCheck, reproduce_failure
+from hypothesis import assume, settings, example
 from hypothesis import strategies as hyst
 
 from pymor.algorithms.basic import almost_equal
+from pymor.core.config import config
 from pymor.vectorarrays.interface import VectorSpace
-from pymor.tools import floatcmp
-from pymor.tools.floatcmp import float_cmp
+from pymor.vectorarrays.numpy import NumpyVectorSpace
+from pymor.tools.floatcmp import float_cmp, bounded
 from pymortests.pickling import assert_picklable_without_dumps_function
 import pymortests.strategies as pyst
-from pymortests.strategies import invalid_inds
+
+MAX_RNG_REALIZATIONS = 30
 
 
 def ind_complement(v, ind):
@@ -114,7 +115,7 @@ def test_full(vector_array):
     with pytest.raises(Exception):
         vector_array.full(9, -1)
     for c in (0, 1, 2, 30):
-        for val in (-1e-3,0,7):
+        for val in (-1e-3, 0, 7):
             v = vector_array.full(val, count=c)
             assert v.space == vector_array.space
             assert len(v) == c
@@ -128,10 +129,31 @@ def test_full(vector_array):
                 pass
 
 
-@pyst.given_vector_arrays(realizations=hyst.integers(min_value=0, max_value=30),
+@pyst.given_vector_arrays(realizations=hyst.integers(min_value=0, max_value=MAX_RNG_REALIZATIONS),
                           low=hyst.floats(allow_infinity=False, allow_nan=False),
                           high=hyst.floats(allow_infinity=False, allow_nan=False))
-def test_random_uniform(vector_array, realizations, low, high):
+@example(vector_array=NumpyVectorSpace(1).empty(), realizations=2,
+         low=-5e-324, high=0.0)
+def test_random_uniform_all(vector_array, realizations, low, high):
+    if config.HAVE_DUNEGDT:
+        # atm needs special casing due to norm implemenation handling of large vector elements
+        from pymor.bindings.dunegdt import DuneXTVectorSpace
+        assume(not isinstance(vector_array.space, DuneXTVectorSpace))
+    _test_random_uniform(vector_array, realizations, low, high)
+
+
+if config.HAVE_DUNEGDT:
+    @pyst.given_vector_arrays(realizations=hyst.integers(min_value=0, max_value=MAX_RNG_REALIZATIONS),
+                              low=hyst.floats(allow_infinity=False, allow_nan=False,
+                                              max_value=10e100, min_value=-10e100),
+                              high=hyst.floats(allow_infinity=False, allow_nan=False,
+                                               max_value=10e100, min_value=-10e100),
+                              which=('dunegdt',))
+    def test_random_uniform_dune(vector_array, realizations, low, high):
+        _test_random_uniform(vector_array, realizations, low, high)
+
+
+def _test_random_uniform(vector_array, realizations, low, high):
     # avoid Overflow in np.random.RandomState.uniform
     assume(np.isfinite(high-low))
     with pytest.raises(Exception):
@@ -151,11 +173,11 @@ def test_random_uniform(vector_array, realizations, low, high):
     assert v.space == vector_array.space
     assert len(v) == c
     if min(v.dim, c) > 0:
-        assert np.all(v.sup_norm() < max(abs(low), abs(high)))
+        assert np.all(v.sup_norm() <= max(abs(low), abs(high)))
     try:
         x = v.to_numpy()
         assert x.shape == (c, v.dim)
-        assert np.all(x < high)
+        assert np.all(x <= high)
         assert np.all(x >= low)
     except NotImplementedError:
         pass
@@ -163,16 +185,14 @@ def test_random_uniform(vector_array, realizations, low, high):
     assert np.allclose((v - vv).sup_norm(), 0.)
 
 
-@settings(deadline=None)
 @pyst.given_vector_arrays(realizations=hyst.integers(min_value=0, max_value=30),
                           loc=hyst.floats(allow_infinity=False, allow_nan=False),
                           scale=hyst.floats(allow_infinity=False, allow_nan=False))
-# TODO the first call to this test is magnitudes slower than the next
 def test_random_normal(vector_array, realizations, loc, scale):
     with pytest.raises(Exception):
         vector_array.random(-1)
     c = realizations
-    if c > 0 and scale < 0:
+    if c > 0 > scale:
         with pytest.raises(ValueError):
             vector_array.random(c, 'normal', loc=loc, scale=scale)
         return
@@ -198,7 +218,7 @@ def test_random_normal(vector_array, realizations, loc, scale):
         alpha = 1 - gamma
         lower = np.sum(x)/n - norm.ppf(1 - alpha/2) * scale / np.sqrt(n)
         upper = np.sum(x)/n + norm.ppf(1 - alpha/2) * scale / np.sqrt(n)
-        floatcmp.bounded(lower, upper, loc)
+        bounded(lower, upper, loc)
     except NotImplementedError:
         pass
     vv = vector_array.random(c, 'normal', loc=loc, scale=scale, seed=seed)
@@ -270,6 +290,28 @@ def test_copy(vectors_and_indices):
             pass
 
 
+@pyst.given_vector_arrays(index_strategy=pyst.valid_indices)
+@example(vectors_and_indices=(NumpyVectorSpace(1).full(2.22044605e-16, 1), [0]))
+def test_COW(vectors_and_indices):
+    v, ind = vectors_and_indices
+    for deep in (True, False):
+        if ind is None:
+            c = v.copy(deep)
+            assert len(c) == len(v)
+        else:
+            c = v[ind].copy(deep)
+            assert len(c) == v.len_ind(ind)
+        assert c.space == v.space
+        if len(c) > 0 and not np.all(c.norm() == 0):
+            c *= 2
+            vi = v[ind] if ind else v
+            assert not np.all(almost_equal(c, vi, atol=0, rtol=0))
+            try:
+                assert np.allclose(c.to_numpy(), 2*indexed(v.to_numpy(), ind))
+            except NotImplementedError:
+                pass
+
+
 @pyst.given_vector_arrays()
 def test_copy_repeated_index(vector_array):
     v = vector_array
@@ -294,33 +336,30 @@ def test_copy_repeated_index(vector_array):
             pass
 
 
-@pyst.given_vector_arrays(count=2)
-# TODO replace indices loop
-@settings(deadline=None)
-def test_append(vector_arrays):
-    v1, v2 = vector_arrays
-    len_v1, len_v2 = len(v1), len(v2)
-    for ind in pyst.valid_inds(v2, random_module=False):
-        c1, c2 = v1.copy(), v2.copy()
-        c1.append(c2[ind])
-        len_ind = v2.len_ind(ind)
-        ind_complement_ = ind_complement(v2, ind)
-        assert len(c1) == len_v1 + len_ind
-        assert np.all(almost_equal(c1[len_v1:len(c1)], c2[ind]))
-        try:
-            assert np.allclose(c1.to_numpy(), np.vstack((v1.to_numpy(), indexed(v2.to_numpy(), ind))))
-        except NotImplementedError:
-            pass
-        c1.append(c2[ind], remove_from_other=True)
-        assert len(c2) == len(ind_complement_)
-        assert c2.space == c1.space
-        assert len(c1) == len_v1 + 2 * len_ind
-        assert np.all(almost_equal(c1[len_v1:len_v1 + len_ind], c1[len_v1 + len_ind:len(c1)]))
-        assert np.all(almost_equal(c2, v2[ind_complement_]))
-        try:
-            assert np.allclose(c2.to_numpy(), indexed(v2.to_numpy(), ind_complement_))
-        except NotImplementedError:
-            pass
+@pyst.given_vector_arrays(count=2, index_strategy=pyst.pairs_both_lengths)
+def test_append(vectors_and_indices):
+    (v1, v2), (_, ind) = vectors_and_indices
+    len_v1 = len(v1)
+    c1, c2 = v1.copy(), v2.copy()
+    c1.append(c2[ind])
+    len_ind = v2.len_ind(ind)
+    ind_complement_ = ind_complement(v2, ind)
+    assert len(c1) == len_v1 + len_ind
+    assert np.all(almost_equal(c1[len_v1:len(c1)], c2[ind]))
+    try:
+        assert np.allclose(c1.to_numpy(), np.vstack((v1.to_numpy(), indexed(v2.to_numpy(), ind))))
+    except NotImplementedError:
+        pass
+    c1.append(c2[ind], remove_from_other=True)
+    assert len(c2) == len(ind_complement_)
+    assert c2.space == c1.space
+    assert len(c1) == len_v1 + 2 * len_ind
+    assert np.all(almost_equal(c1[len_v1:len_v1 + len_ind], c1[len_v1 + len_ind:len(c1)]))
+    assert np.all(almost_equal(c2, v2[ind_complement_]))
+    try:
+        assert np.allclose(c2.to_numpy(), indexed(v2.to_numpy(), ind_complement_))
+    except NotImplementedError:
+        pass
 
 
 @pyst.given_vector_arrays()
@@ -357,7 +396,7 @@ def test_del(vectors_and_indices):
 
 
 @pyst.given_vector_arrays(index_strategy=pyst.valid_indices)
-def test_scla(vectors_and_indices):
+def test_scal(vectors_and_indices):
     v, ind = vectors_and_indices
     if v.len_ind(ind) != v.len_ind_unique(ind):
         with pytest.raises(Exception):
@@ -391,58 +430,64 @@ def test_scla(vectors_and_indices):
             pass
 
 
-@pyst.given_vector_arrays(count=2, random=hyst.random_module())
-# TODO replace indices loop
-@settings(deadline=None)
-def test_axpy(vector_arrays, random):
-    v1, v2 = vector_arrays
+@pyst.given_vector_arrays()
+def test_scal_imaginary(vector_array):
+    v = vector_array
+    w = v.copy()
+    w.scal(1j)
+    assert np.allclose(v.norm(), w.norm())
 
-    for ind1, ind2 in pyst.valid_inds_of_same_length(v1, v2, random_module=False):
-        if v1.len_ind(ind1) != v1.len_ind_unique(ind1):
-            with pytest.raises(Exception):
-                c1, c2 = v1.copy(), v2.copy()
-                c1[ind1].axpy(0., c2[ind2])
-            continue
 
-        ind1_complement = ind_complement(v1, ind1)
-        c1, c2 = v1.copy(), v2.copy()
-        c1[ind1].axpy(0., c2[ind2])
-        assert len(c1) == len(v1)
-        assert np.all(almost_equal(c1, v1))
-        assert np.all(almost_equal(c2, v2))
-
-        for a in (1., 1.4, np.random.random(v1.len_ind(ind1))):
+@pyst.given_vector_arrays(count=2, index_strategy=pyst.pairs_same_length,
+                          scalar=hyst.floats(min_value=1, max_value=pyst.MAX_VECTORARRAY_LENGTH))
+def test_axpy(vectors_and_indices, scalar):
+    (v1, v2), (ind1, ind2) = vectors_and_indices
+    if v1.len_ind(ind1) != v1.len_ind_unique(ind1):
+        with pytest.raises(Exception):
             c1, c2 = v1.copy(), v2.copy()
-            c1[ind1].axpy(a, c2[ind2])
-            assert len(c1) == len(v1)
-            assert np.all(almost_equal(c1[ind1_complement], v1[ind1_complement]))
-            assert np.all(almost_equal(c2, v2))
-            assert np.all(c1[ind1].sup_norm() <= v1[ind1].sup_norm() + abs(a) * v2[ind2].sup_norm() * (1. + 1e-10))
-            assert np.all(c1[ind1].norm() <= (v1[ind1].norm() + abs(a) * v2[ind2].norm()) * (1. + 1e-10))
-            try:
-                x = v1.to_numpy(True).astype(complex)  # ensure that inplace addition works
-                if isinstance(ind1, Number):
-                    x[[ind1]] += indexed(v2.to_numpy(), ind2) * a
-                else:
-                    if isinstance(a, np.ndarray):
-                        aa = a[:, np.newaxis]
-                    else:
-                        aa = a
-                    x[ind1] += indexed(v2.to_numpy(), ind2) * aa
-                assert np.allclose(c1.to_numpy(), x)
-            except NotImplementedError:
-                pass
-            c1[ind1].axpy(-a, c2[ind2])
-            assert len(c1) == len(v1)
-            assert np.all(almost_equal(c1, v1))
+            c1[ind1].axpy(0., c2[ind2])
+        return
+    # ind2 is used for axpy args
+    len_ind2 = v2.len_ind(ind2)
+    assume(len_ind2 == 1 or len_ind2 == v1.len_ind(ind1))
+    ind1_complement = ind_complement(v1, ind1)
+    c1, c2 = v1.copy(), v2.copy()
+    c1[ind1].axpy(0., c2[ind2])
+    assert len(c1) == len(v1)
+    assert np.all(almost_equal(c1, v1))
+    assert np.all(almost_equal(c2, v2))
+
+    a = scalar
+    c1, c2 = v1.copy(), v2.copy()
+    c1[ind1].axpy(a, c2[ind2])
+    assert len(c1) == len(v1)
+    assert np.all(almost_equal(c1[ind1_complement], v1[ind1_complement]))
+    assert np.all(almost_equal(c2, v2))
+    assert np.all(c1[ind1].sup_norm() <= v1[ind1].sup_norm() + abs(a) * v2[ind2].sup_norm() * (1. + 1e-10))
+    assert np.all(c1[ind1].norm() <= (v1[ind1].norm() + abs(a) * v2[ind2].norm()) * (1. + 1e-10))
+    try:
+        x = v1.to_numpy(True).astype(complex)  # ensure that inplace addition works
+        if isinstance(ind1, Number):
+            x[[ind1]] += indexed(v2.to_numpy(), ind2) * a
+        else:
+            if isinstance(a, np.ndarray):
+                aa = a[:, np.newaxis]
+            else:
+                aa = a
+            x[ind1] += indexed(v2.to_numpy(), ind2) * aa
+        assert np.allclose(c1.to_numpy(), x)
+    except NotImplementedError:
+        pass
+    c1[ind1].axpy(-a, c2[ind2])
+    assert len(c1) == len(v1)
+    assert np.all(almost_equal(c1, v1, atol=1e-13, rtol=1e-13))
 
 
-@pyst.given_vector_arrays(count=2, random=hyst.random_module())
-# TODO replace indices loop
-@settings(deadline=None)
-def test_axpy_one_x(vector_arrays, random):
-    v1, v2 = vector_arrays
-    for ind1, ind2 in product(pyst.valid_inds(v1, random_module=False), pyst.valid_inds(v2, 1, random_module=False)):
+@pyst.given_vector_arrays(count=2, index_strategy=pyst.pairs_same_length,
+                          scalar=hyst.floats(min_value=1, max_value=pyst.MAX_VECTORARRAY_LENGTH))
+def test_axpy_one_x(vectors_and_indices, scalar):
+    (v1, v2), (ind1, _) = vectors_and_indices
+    for ind2 in pyst.valid_inds(v2, 1, random_module=False):
         assert v1.check_ind(ind1)
         assert v2.check_ind(ind2)
         if v1.len_ind(ind1) != v1.len_ind_unique(ind1):
@@ -456,47 +501,43 @@ def test_axpy_one_x(vector_arrays, random):
 
         gc = c1[ind1]
         gv = c2[ind2]
-        # TODO this was somehow hardcoded in the old fixture, makes the test extremely slow
-        assume(len(gc) == 0 or len(gv) > 0 )
         gc.axpy(0., gv)
         assert len(c1) == len(v1)
         assert np.all(almost_equal(c1, v1))
         assert np.all(almost_equal(c2, v2))
 
-        for a in (1., 1.4, np.random.random(v1.len_ind(ind1))):
-            c1, c2 = v1.copy(), v2.copy()
-            c1[ind1].axpy(a, c2[ind2])
-            assert len(c1) == len(v1)
-            assert np.all(almost_equal(c1[ind1_complement], v1[ind1_complement]))
-            assert np.all(almost_equal(c2, v2))
-            # for the openstack CI machines this could be 1 + 1e-10
-            rtol_factor = 1. + 147e-9
-            assert np.all(c1[ind1].sup_norm() <=  v1[ind1].sup_norm() + abs(a) * v2[ind2].sup_norm() * rtol_factor)
-            assert np.all(c1[ind1].norm() <= (v1[ind1].norm() + abs(a) * v2[ind2].norm()) * (1. + 1e-10))
-            try:
-                x = v1.to_numpy(True).astype(complex)  # ensure that inplace addition works
-                if isinstance(ind1, Number):
-                    x[[ind1]] += indexed(v2.to_numpy(), ind2) * a
+        a = scalar
+        c1, c2 = v1.copy(), v2.copy()
+        c1[ind1].axpy(a, c2[ind2])
+        assert len(c1) == len(v1)
+        assert np.all(almost_equal(c1[ind1_complement], v1[ind1_complement]))
+        assert np.all(almost_equal(c2, v2))
+        # for the openstack CI machines this could be 1 + 1e-10
+        rtol_factor = 1. + 147e-9
+        assert np.all(c1[ind1].sup_norm() <= v1[ind1].sup_norm() + abs(a) * v2[ind2].sup_norm() * rtol_factor)
+        assert np.all(c1[ind1].norm() <= (v1[ind1].norm() + abs(a) * v2[ind2].norm()) * (1. + 1e-10))
+        try:
+            x = v1.to_numpy(True).astype(complex)  # ensure that inplace addition works
+            if isinstance(ind1, Number):
+                x[[ind1]] += indexed(v2.to_numpy(), ind2) * a
+            else:
+                if isinstance(a, np.ndarray):
+                    aa = a[:, np.newaxis]
                 else:
-                    if isinstance(a, np.ndarray):
-                        aa = a[:, np.newaxis]
-                    else:
-                        aa = a
-                    x[ind1] += indexed(v2.to_numpy(), ind2) * aa
-                assert np.allclose(c1.to_numpy(), x)
-            except NotImplementedError:
-                pass
-            c1[ind1].axpy(-a, c2[ind2])
-            assert len(c1) == len(v1)
-            assert np.all(almost_equal(c1, v1))
+                    aa = a
+                x[ind1] += indexed(v2.to_numpy(), ind2) * aa
+            assert np.allclose(c1.to_numpy(), x)
+        except NotImplementedError:
+            pass
+        c1[ind1].axpy(-a, c2[ind2])
+        assert len(c1) == len(v1)
+        assert np.all(almost_equal(c1, v1, atol=1e-13, rtol=1e-13))
 
 
-@pyst.given_vector_arrays(index_strategy=pyst.pairs_same_length , random=hyst.random_module())
-# TODO replace indices loop
-@settings(deadline=None)
-def test_axpy_self(vectors_and_indices, random):
+@pyst.given_vector_arrays(index_strategy=pyst.pairs_same_length,
+                          scalar=hyst.floats(min_value=1, max_value=pyst.MAX_VECTORARRAY_LENGTH))
+def test_axpy_self(vectors_and_indices, scalar):
     v, (ind1, ind2) = vectors_and_indices
-
     if v.len_ind(ind1) != v.len_ind_unique(ind1):
         with pytest.raises(Exception):
             c, = v.copy()
@@ -506,33 +547,32 @@ def test_axpy_self(vectors_and_indices, random):
     ind1_complement = ind_complement(v, ind1)
     c = v.copy()
     rr = c[ind2]
-    lp =c[ind1]
+    lp = c[ind1]
     lp.axpy(0., rr)
     assert len(c) == len(v)
     assert np.all(almost_equal(c, v))
-
-    for a in (1., 1.4, np.random.random(v.len_ind(ind1))):
-        c = v.copy()
-        c[ind1].axpy(a, c[ind2])
-        assert len(c) == len(v)
-        assert np.all(almost_equal(c[ind1_complement], v[ind1_complement]))
-        assert np.all(c[ind1].sup_norm() <= v[ind1].sup_norm() + abs(a) * v[ind2].sup_norm() * (1. + 1e-10))
-        try:
-            x = v.to_numpy(True).astype(complex)  # ensure that inplace addition works
-            if isinstance(ind1, Number):
-                x[[ind1]] += indexed(v.to_numpy(), ind2) * a
+    a = scalar
+    c = v.copy()
+    c[ind1].axpy(a, c[ind2])
+    assert len(c) == len(v)
+    assert np.all(almost_equal(c[ind1_complement], v[ind1_complement]))
+    assert np.all(c[ind1].sup_norm() <= v[ind1].sup_norm() + abs(a) * v[ind2].sup_norm() * (1. + 1e-10))
+    try:
+        x = v.to_numpy(True).astype(complex)  # ensure that inplace addition works
+        if isinstance(ind1, Number):
+            x[[ind1]] += indexed(v.to_numpy(), ind2) * a
+        else:
+            if isinstance(a, np.ndarray):
+                aa = a[:, np.newaxis]
             else:
-                if isinstance(a, np.ndarray):
-                    aa = a[:, np.newaxis]
-                else:
-                    aa = a
-                x[ind1] += indexed(v.to_numpy(), ind2) * aa
-            assert np.allclose(c.to_numpy(), x)
-        except NotImplementedError:
-            pass
-        c[ind1].axpy(-a, v[ind2])
-        assert len(c) == len(v)
-        assert np.all(almost_equal(c, v))
+                aa = a
+            x[ind1] += indexed(v.to_numpy(), ind2) * aa
+        assert np.allclose(c.to_numpy(), x)
+    except NotImplementedError:
+        pass
+    c[ind1].axpy(-a, v[ind2])
+    assert len(c) == len(v)
+    assert np.all(almost_equal(c, v))
 
     ind = ind1
     if v.len_ind(ind) != v.len_ind_unique(ind):
@@ -555,7 +595,7 @@ def test_pairwise_inner(vector_arrays):
         assert r.shape == (v1.len_ind(ind1),)
         r2 = v2[ind2].pairwise_inner(v1[ind1])
         assert np.allclose, (r, r2)
-        assert np.all(r <= v1[ind1].norm() * v2[ind2].norm() * (1. + 1e-10))
+        assert np.all(r <= (v1[ind1].norm() * v2[ind2].norm() * (1. + 1e-10) + 1e-15))
         try:
             assert np.allclose(r, np.sum(indexed(v1.to_numpy(), ind1).conj() * indexed(v2.to_numpy(), ind2), axis=1))
         except NotImplementedError:
@@ -570,9 +610,9 @@ def test_pairwise_inner_self(vectors_and_indices):
     assert r.shape == (v.len_ind(ind1),)
     r2 = v[ind2].pairwise_inner(v[ind1])
     assert np.allclose(r, r2.T.conj())
-    assert np.all(r <= v[ind1].norm() * v[ind2].norm() * (1. + 1e-10))
+    assert np.all(r <= (v[ind1].norm() * v[ind2].norm() * (1. + 1e-10) + 1e-15))
     try:
-            assert np.allclose(r, np.sum(indexed(v.to_numpy(), ind1).conj() * indexed(v.to_numpy(), ind2), axis=1))
+        assert np.allclose(r, np.sum(indexed(v.to_numpy(), ind1).conj() * indexed(v.to_numpy(), ind2), axis=1))
     except NotImplementedError:
         pass
     ind = ind1
@@ -583,35 +623,19 @@ def test_pairwise_inner_self(vectors_and_indices):
 @settings(deadline=None, print_blob=True)
 @pyst.given_vector_arrays(count=2, index_strategy=pyst.pairs_both_lengths)
 def test_inner(vectors_and_indices):
-    vectors, indices = vectors_and_indices
-    v1, v2 = vectors
-    ind1, ind2 = indices
-
-    # TODO
-    assume_old_slicing(indices)
-
+    (v1, v2), (ind1, ind2) = vectors_and_indices
     r = v1[ind1].inner(v2[ind2])
     assert isinstance(r, np.ndarray)
     assert r.shape == (v1.len_ind(ind1), v2.len_ind(ind2))
     r2 = v2[ind2].inner(v1[ind1])
     assert np.allclose(r, r2.T.conj())
-    assert np.all(r <= v1[ind1].norm()[:, np.newaxis] * v2[ind2].norm()[np.newaxis, :] * (1. + 1e-10))
+    assert np.all(r <= (v1[ind1].norm()[:, np.newaxis] * v2[ind2].norm()[np.newaxis, :] * (1. + 1e-10) + 1e-15))
     try:
         assert np.allclose(r, indexed(v1.to_numpy(), ind1).conj().dot(indexed(v2.to_numpy(), ind2).T))
     except NotImplementedError:
         pass
 
 
-def assume_old_slicing(indices):
-    # TODO old data input did not have None as X in slices, only slice(None)
-    for ind in indices:
-        if ind is slice(None):
-            continue
-        if isinstance(ind, slice):
-            for p in ('step', 'start', 'stop'):
-                assume(getattr(ind, p) is not None)
-
-
 @settings(deadline=None)
 @pyst.given_vector_arrays(index_strategy=pyst.pairs_both_lengths)
 def test_inner_self(vectors_and_indices):
@@ -621,9 +645,9 @@ def test_inner_self(vectors_and_indices):
     assert r.shape == (v.len_ind(ind1), v.len_ind(ind2))
     r2 = v[ind2].inner(v[ind1])
     assert np.allclose(r, r2.T.conj())
-    assert np.all(r <= v[ind1].norm()[:, np.newaxis] * v[ind2].norm()[np.newaxis, :] * (1. + 1e-10))
+    assert np.all(r <= (v[ind1].norm()[:, np.newaxis] * v[ind2].norm()[np.newaxis, :] * (1. + 1e-10) + 1e-15))
     try:
-            assert np.allclose(r, indexed(v.to_numpy(), ind1).conj().dot(indexed(v.to_numpy(), ind2).T))
+        assert np.allclose(r, indexed(v.to_numpy(), ind1).conj().dot(indexed(v.to_numpy(), ind2).T))
     except NotImplementedError:
         pass
     r = v[ind1].inner(v[ind1])
@@ -686,7 +710,7 @@ def test_norm(vectors_and_indices):
     if v.dim == 0:
         assert np.all(norm == 0)
     try:
-            assert np.allclose(norm, np.linalg.norm(indexed(v.to_numpy(), ind), axis=1))
+        assert np.allclose(norm, np.linalg.norm(indexed(v.to_numpy(), ind), axis=1))
     except NotImplementedError:
         pass
     c.scal(4.)
@@ -708,7 +732,7 @@ def test_norm2(vectors_and_indices):
     if v.dim == 0:
         assert np.all(norm == 0)
     try:
-            assert np.allclose(norm, np.linalg.norm(indexed(v.to_numpy(), ind), axis=1)**2)
+        assert np.allclose(norm, np.linalg.norm(indexed(v.to_numpy(), ind), axis=1)**2)
     except NotImplementedError:
         pass
     c.scal(4.)
@@ -746,7 +770,7 @@ def test_sup_norm(vectors_and_indices):
 def test_dofs(vectors_and_indices, random_count):
     v, ind = vectors_and_indices
     c = v.copy()
-    dofs = c[ind].dofs(np.array([], dtype=np.int))
+    dofs = c[ind].dofs(np.array([], dtype=int))
     assert isinstance(dofs, np.ndarray)
     assert dofs.shape == (v.len_ind(ind), 0)
 
@@ -809,7 +833,6 @@ def test_amax(vectors_and_indices):
 
 
 @pyst.given_vector_arrays(index_strategy=pyst.valid_indices)
-@settings(deadline=None)
 def test_gramian(vectors_and_indices):
     v, ind = vectors_and_indices
     assert np.allclose(v[ind].gramian(), v[ind].inner(v[ind]))
@@ -862,16 +885,14 @@ def test_neg(vector_array):
     assert np.all(almost_equal(vector_array, cc))
 
 
-@pyst.given_vector_arrays()
-# TODO split and replace count loop
-@settings(deadline=None)
-def test_mul(vector_array):
+@pyst.given_vector_arrays(index_strategy=pyst.st_scaling_value)
+def test_mul(vectors_and_indices):
+    vector_array, a = vectors_and_indices
     c = vector_array.copy()
-    for a in (-1, -3, 0, 1, 23, np.arange(len(vector_array))):
-        cc = vector_array.copy()
-        cc.scal(a)
-        assert np.all(almost_equal((vector_array * a), cc))
-        assert np.all(almost_equal(vector_array, c))
+    cc = vector_array.copy()
+    cc.scal(a)
+    assert np.all(almost_equal((vector_array * a), cc))
+    assert np.all(almost_equal(vector_array, c))
 
 
 @pyst.given_vector_arrays()
@@ -880,28 +901,28 @@ def test_mul_wrong_factor(vector_array):
         _ = vector_array * vector_array
 
 
-@pyst.given_vector_arrays()
-# TODO split and replace count loop
-@settings(deadline=None)
-def test_rmul(vector_array):
+@pyst.given_vector_arrays(index_strategy=pyst.st_scaling_value)
+def test_rmul(vectors_and_indices):
+    vector_array, a = vectors_and_indices
     c = vector_array.copy()
-    for a in (-1, -3, 0, 1, 23):
-        cc = vector_array.copy()
-        cc.scal(a)
-        assert np.all(almost_equal((a * vector_array), cc))
-        assert np.all(almost_equal(vector_array, c))
+    cc = vector_array.copy()
+    cc.scal(a)
+    alpha = a * vector_array
+    # the scaling_value strategy also draws ndarrays, for which alpha here will be an ndarray,
+    # which in turn will fail the axpy hidden in the almost_equal check
+    assume(not isinstance(alpha, np.ndarray))
+    assert np.all(almost_equal(alpha, cc))
+    assert np.all(almost_equal(vector_array, c))
 
 
-@pyst.given_vector_arrays()
-# TODO split and replace count loop
-@settings(deadline=None)
-def test_imul(vector_array):
-    for a in (-1, -3, 0, 1, 23, np.arange(len(vector_array))):
-        c = vector_array.copy()
-        cc = vector_array.copy()
-        c.scal(a)
-        cc *= a
-        assert np.all(almost_equal(c, cc))
+@pyst.given_vector_arrays(index_strategy=pyst.st_scaling_value)
+def test_imul(vectors_and_indices):
+    vector_array, a = vectors_and_indices
+    c = vector_array.copy()
+    cc = vector_array.copy()
+    c.scal(a)
+    cc *= a
+    assert np.all(almost_equal(c, cc))
 
 
 @pyst.given_vector_arrays()
@@ -909,6 +930,7 @@ def test_imul_wrong_factor(vector_array):
     with pytest.raises(Exception):
         vector_array *= vector_array
 
+
 @pyst.given_vector_arrays()
 def test_iter(vector_array):
     v = vector_array
@@ -917,7 +939,9 @@ def test_iter(vector_array):
         w.append(vv)
     assert np.all(almost_equal(w, v))
 
-########################################################################################################################
+
+####################################################################################################
+
 
 @pyst.given_vector_arrays(count=2, compatible=False)
 def test_append_incompatible(vector_arrays):
@@ -994,15 +1018,14 @@ def test_isub_incompatible(vector_arrays):
         v1 -= v2
 
 
-########################################################################################################################
+####################################################################################################
 
 
-@pyst.given_vector_arrays()
-def test_wrong_ind_raises_exception(vector_array):
-    # TODO index input as hypothesis strategy
-    for ind in invalid_inds(vector_array):
-        with pytest.raises(Exception):
-            vector_array[ind]
+@pyst.given_vector_arrays(index_strategy=pyst.invalid_indices)
+def test_wrong_ind_raises_exception(vectors_and_indices):
+    vector_array, ind = vectors_and_indices
+    with pytest.raises(Exception):
+        vector_array[ind]
 
 
 @pyst.given_vector_arrays(index_strategy=pyst.valid_indices)
@@ -1015,16 +1038,14 @@ def test_scal_wrong_coefficients(vectors_and_indices):
             v[ind].scal(alpha)
 
 
-@pyst.given_vector_arrays(count=2, random=hyst.random_module())
-def test_axpy_wrong_coefficients(vector_arrays, random):
-    v1, v2 = vector_arrays
-    # TODO data input from hypothesis strategy
-    for ind1, ind2 in pyst.valid_inds_of_same_length(v1, v2):
-        for alpha in ([np.array([]), np.eye(v1.len_ind(ind1)), np.random.random(v1.len_ind(ind1) + 1)]
-                      if v1.len_ind(ind1) > 0 else
-                      [np.random.random(1)]):
-            with pytest.raises(Exception):
-                v1[ind1].axpy(alpha, v2[ind2])
+@pyst.given_vector_arrays(count=2, index_strategy=pyst.pairs_same_length)
+def test_axpy_wrong_coefficients(vectors_and_indices):
+    (v1, v2), (ind1, ind2) = vectors_and_indices
+    for alpha in ([np.array([]), np.eye(v1.len_ind(ind1)), np.random.random(v1.len_ind(ind1) + 1)]
+                  if v1.len_ind(ind1) > 0 else
+                  [np.random.random(1)]):
+        with pytest.raises(Exception):
+            v1[ind1].axpy(alpha, v2[ind2])
 
 
 @pyst.given_vector_arrays(which='picklable')
